==> blender_3.2.2+dfsg.orig.tar.xz: blender-3.2.2/source/blender/compositor/operations/COM_CryptomatteOperation.cc <==
/* SPDX-License-Identifier: GPL-2.0-or-later
 * Copyright 2018 Blender Foundation. */

#include "COM_CryptomatteOperation.h"

namespace blender::compositor {

CryptomatteOperation::CryptomatteOperation(size_t num_inputs)
{
  inputs.resize(num_inputs);
  for (size_t i = 0; i < num_inputs; i++) {
    this->add_input_socket(DataType::Color);
  }
  this->add_output_socket(DataType::Color);
  flags_.complex = true;
}

void CryptomatteOperation::init_execution()
{
  for (size_t i = 0; i < inputs.size(); i++) {
    inputs[i] = this->get_input_socket_reader(i);
  }
}

void CryptomatteOperation::add_object_index(float object_index)
{
  if (object_index != 0.0f) {
    object_index_.append(object_index);
  }
}

void CryptomatteOperation::execute_pixel(float output[4], int x, int y, void *data)
{
  float input[4];
  output[0] = output[1] = output[2] = output[3] = 0.0f;
  for (size_t i = 0; i < inputs.size(); i++) {
    inputs[i]->read(input, x, y, data);
    if (i == 0) {
      /* Write the front-most object as false color for picking. */
      output[0] = input[0];
      uint32_t m3hash;
      ::memcpy(&m3hash, &input[0], sizeof(uint32_t));
      /* Since the red channel is likely to be out of display range,
       * setting green and blue gives more meaningful images. */
      output[1] = ((float)(m3hash << 8) / (float)UINT32_MAX);
      output[2] = ((float)(m3hash << 16) / (float)UINT32_MAX);
    }
    for (float hash : object_index_) {
      if (input[0] == hash) {
        output[3] += input[1];
      }
      if (input[2] == hash) {
        output[3] += input[3];
      }
    }
  }
}

void CryptomatteOperation::update_memory_buffer_partial(MemoryBuffer *output,
                                                        const rcti &area,
                                                        Span<MemoryBuffer *> inputs)
{
  for (BuffersIterator<float> it = output->iterate_with(inputs, area); !it.is_end(); ++it) {
    zero_v4(it.out);
    for (int i = 0; i < it.get_num_inputs(); i++) {
      const float *input = it.in(i);
      if (i == 0) {
        /* Write the front-most object as false color for picking. */
        it.out[0] = input[0];
        uint32_t m3hash;
        ::memcpy(&m3hash, &input[0], sizeof(uint32_t));
        /* Since the red channel is likely to be out of display range,
         * setting green and blue gives more meaningful images. */
        it.out[1] = ((float)(m3hash << 8) / (float)UINT32_MAX);
        it.out[2] = ((float)(m3hash << 16) / (float)UINT32_MAX);
      }
      for (const float hash : object_index_) {
        if (input[0] == hash) {
          it.out[3] += input[1];
        }
        if (input[2] == hash) {
          it.out[3] += input[3];
        }
      }
    }
  }
}

}  // namespace blender::compositor

==> blender_3.4.1+dfsg.orig.tar.xz: blender-3.4.1/source/blender/compositor/operations/COM_CryptomatteOperation.cc <==
/* SPDX-License-Identifier: GPL-2.0-or-later
 * Copyright 2018 Blender Foundation. */

#include "COM_CryptomatteOperation.h"

namespace blender::compositor {

CryptomatteOperation::CryptomatteOperation(size_t num_inputs)
{
  inputs.resize(num_inputs);
  for (size_t i = 0; i < num_inputs; i++) {
    this->add_input_socket(DataType::Color);
  }
  this->add_output_socket(DataType::Color);
  flags_.complex = true;
}

void CryptomatteOperation::init_execution()
{
  for (size_t i = 0; i < inputs.size(); i++) {
    inputs[i] = this->get_input_socket_reader(i);
  }
}

void CryptomatteOperation::add_object_index(float object_index)
{
  if (object_index != 0.0f) {
    object_index_.append(object_index);
  }
}

void CryptomatteOperation::execute_pixel(float output[4], int x, int y, void *data)
{
  float input[4];
  output[0] = output[1] = output[2] = output[3] = 0.0f;
  for (size_t i = 0; i < inputs.size(); i++) {
    inputs[i]->read(input, x, y, data);
    if (i == 0) {
      /* Write the front-most object as false color for picking. */
      output[0] = input[0];
      uint32_t m3hash;
      ::memcpy(&m3hash, &input[0], sizeof(uint32_t));
      /* Since the red channel is likely to be out of display range,
       * setting green and blue gives more meaningful images. */
      output[1] = (float(m3hash << 8) / float(UINT32_MAX));
      output[2] = (float(m3hash << 16) / float(UINT32_MAX));
    }
    for (float hash : object_index_) {
      if (input[0] == hash) {
        output[3] += input[1];
      }
      if (input[2] == hash) {
        output[3] += input[3];
      }
    }
  }
}

void CryptomatteOperation::update_memory_buffer_partial(MemoryBuffer *output,
                                                        const rcti &area,
                                                        Span<MemoryBuffer *> inputs)
{
  for (BuffersIterator<float> it = output->iterate_with(inputs, area); !it.is_end(); ++it) {
    zero_v4(it.out);
    for (int i = 0; i < it.get_num_inputs(); i++) {
      const float *input = it.in(i);
      if (i == 0) {
        /* Write the front-most object as false color for picking. */
        it.out[0] = input[0];
        uint32_t m3hash;
        ::memcpy(&m3hash, &input[0], sizeof(uint32_t));
        /* Since the red channel is likely to be out of display range,
         * setting green and blue gives more meaningful images. */
        it.out[1] = (float(m3hash << 8) / float(UINT32_MAX));
        it.out[2] = (float(m3hash << 16) / float(UINT32_MAX));
      }
      for (const float hash : object_index_) {
        if (input[0] == hash) {
          it.out[3] += input[1];
        }
        if (input[2] == hash) {
          it.out[3] += input[3];
        }
      }
    }
  }
}

}  // namespace blender::compositor
